@@ -1,27 +1,55 @@
+/* Copyright (C) 2000,2001,2002 Manuel Amador (Rudd-O)
+   This file is part of Directory administrator.
+
+   Directory administrator is free software; you can redistribute it
+   and/or modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+
+   Directory administrator is distributed in the hope that it will be
+   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+*/
+
+
+#include "charset.h"
 #include <glib.h>
 #include <gnome.h>
 #include "appsupport.h"
 #include "groups.h"
+#include "charset.h"
+#include "dir_entry.h"
+#include "appglobals.h"
+#include "appfunctions.h"
+#include "schema.h"
 
 ///TODAS LAS FUNCIONES RETORNAN REFERENCIAS A SUS ETRUCTURAS INTERNAS, PERO SACAN
 ///COPIAS DE LOS STRINGS PASADOS PARA GUARDARLOS.  ESTOS DATOS SE DESTRUYEN AL LLAMAR
 ///A LAS FUNCIONES DESTROY
 ///OTROS TIPOS DE ESTRUCTURAS COMO LISTAS, NO SE COPIAN, SOLO SE GUARDAN REFERENCIAS
 
+gchar *group_handled_objectclasses[] = {
+"top","groupOfNames","posixGroup", SAMBA_GROUP_MAPPING, NULL
+};
+
+gchar *group_handled_attributes[] = {
+"gidnumber","cn",SAMBA_SID, SAMBA_GROUP_TYPE, NULL
+};
+
 
 void
 g_debug_free (gpointer data)
 {
-  g_print ("\n");
-  g_print ("g_debug_free: freeing element containing ");
-  g_print (data);
-  g_print ("\n");
+  g_print ("\ng_debug_free: freeing element containing %s\n",(char*)data);
   g_free (data);
 }
 
+
+
 diradmin_group *
 diradmin_group_new (gchar * dn,
-		    gchar * cn, gchar * gidnumber, GList * memberlist)
+		    gchar * cn, gchar * gidnumber, gchar *sambaSID)
 {
 
   diradmin_group *conn = NULL;
@@ -29,24 +57,28 @@ diradmin_group_new (gchar * dn,
   conn = g_new (diradmin_group, 1);
   conn->diradmin_group_data = NULL;
   conn->diradmin_group_members = NULL;
+  conn->diradmin_group_dnmembers = NULL;
   conn->diradmin_group_objectclasses = NULL;
 
   diradmin_group_set_attribute (conn, "dn", cn);
   diradmin_group_set_attribute (conn, "cn", cn);
   diradmin_group_set_attribute (conn, "gidnumber", gidnumber);
 
-  diradmin_group_set_members (conn, memberlist);
+  diradmin_group_set_members (conn, NULL);
+  diradmin_group_set_dnmembers (conn, NULL);
 
   conn->diradmin_group_objectclasses =
     g_list_append (conn->diradmin_group_objectclasses, g_strdup ("top"));
   conn->diradmin_group_objectclasses =
     g_list_append (conn->diradmin_group_objectclasses,
 		   g_strdup ("posixGroup"));
-/*  conn->diradmin_group_objectclasses =
-    g_list_append (conn->diradmin_group_objectclasses, g_strdup ("groupOfUniqueNames"));
-
-
-  */
+  if (sambaSID) {
+      diradmin_group_set_attribute (conn, SAMBA_SID, sambaSID);
+      diradmin_group_set_attribute (conn, SAMBA_GROUP_TYPE, DEFAULT_SAMBA_GROUP_TYPE);
+      conn->diradmin_group_objectclasses =
+          g_list_append (conn->diradmin_group_objectclasses,
+  	  g_strdup (SAMBA_GROUP_MAPPING));
+  }
 
   return conn;
 }
@@ -56,15 +88,19 @@ diradmin_group_duplicate (diradmin_group * tobeduped)
 {
 
   diradmin_group *conn = NULL;
-
+g_assert(tobeduped);
   conn = g_new (diradmin_group, 1);
   conn->diradmin_group_data = NULL;
   conn->diradmin_group_members = NULL;
+  conn->diradmin_group_dnmembers = NULL;
 
   conn->diradmin_group_data =
     pairs_list_duplicate (tobeduped->diradmin_group_data);
 
+  if (tobeduped->diradmin_group_members)
   diradmin_group_set_members (conn, tobeduped->diradmin_group_members);
+  if (tobeduped->diradmin_group_dnmembers)
+  diradmin_group_set_dnmembers (conn, tobeduped->diradmin_group_dnmembers);
 
 
   return conn;
@@ -79,6 +115,7 @@ diradmin_group_destroy (diradmin_group * todestroy)
     {
       pairs_list_destroy (todestroy->diradmin_group_data);
       diradmin_group_set_members (todestroy, NULL);
+      diradmin_group_set_dnmembers (todestroy, NULL);
       g_free (todestroy);
     }
 
@@ -117,43 +154,134 @@ diradmin_group_get_members (diradmin_group * conn)
   return g_list_first (conn->diradmin_group_members);
 }
 
+GList *
+diradmin_group_get_dnmembers (diradmin_group * conn)
+{
+  //return a reference to the members list.  shouldnt be manipulated outside.
+  return g_list_first (conn->diradmin_group_dnmembers);
+}
+
 void
 diradmin_group_add_member (diradmin_group * conn, gchar * member)
 {
+
+  dir_entry* userDN = NULL;
+
   diradmin_group_remove_member (conn, member);
   conn->diradmin_group_members =
     g_list_append (conn->diradmin_group_members, g_strdup (member));
-  diradmin_group_add_objectclass (conn, "groupOfUniqueNames");
+
+  userDN = dir_entry_list_getbyuid (cached_dir_entries, member);
+  if (userDN) {
+     g_assert(dir_entry_get_dn(userDN));
+     diradmin_group_add_dnmember(conn,dir_entry_get_dn(userDN));
+  }
+
 }
 
 void
-diradmin_group_remove_member (diradmin_group * conn, gchar * member)
+diradmin_group_add_dnmember (diradmin_group * conn, gchar * member)
 {
-  gchar *data;
-  GList *iterator = NULL;
-  iterator = g_list_first (conn->diradmin_group_members);
-  while (iterator)
-    {
-      data = iterator->data;
-      if (g_strcasecmp (member, iterator->data) == 0)
-	{
-	  g_free (data);
-	  conn->diradmin_group_members =
-	    g_list_remove (conn->diradmin_group_members, iterator->data);
-	}
-      iterator = g_list_next (iterator);
-    }
-  if (g_list_length (conn->diradmin_group_members) == 0)
-    diradmin_group_remove_objectclass (conn, "groupOfUniqueNames");
+  g_assert(member);
+  diradmin_group_remove_dnmember (conn, member);
+  conn->diradmin_group_dnmembers =
+    g_list_append (conn->diradmin_group_dnmembers, g_strdup (member));
+  diradmin_group_add_objectclass (conn, "groupOfNames");
 
 }
 
+/*void diradmin_group_dump_dnmembers(diradmin_group*conn) {
+	GList* loopix;
+	g_print("DN member dump of %s:\n",diradmin_group_get_attribute(conn,"dn"));
+	for (loopix=g_list_first(conn->diradmin_group_members);loopix;loopix=loopix->next) {
+		g_print("    Member: %s\n",loopix->data);
+	}
+
+}
+
+void diradmin_group_dump_members(diradmin_group*conn) {
+	GList* loopix;
+	g_print("Unix member dump of %s:\n",diradmin_group_get_attribute(conn,"dn"));
+	for (loopix=g_list_first(conn->diradmin_group_members);loopix;loopix=loopix->next) {
+		g_print("    Member: %s\n",loopix->data);
+	}
+
+}
+*/
+void
+diradmin_group_remove_member (diradmin_group * conn, gchar * member)
+{
+
+  GList *iterator = NULL;
+  dir_entry* userDN;
+
+  iterator = g_list_find_custom (conn->diradmin_group_members,
+				 member, (GCompareFunc) g_strcasecmp);
+  if (iterator)
+    {
+      conn->diradmin_group_members =
+	g_list_remove_link (conn->diradmin_group_members, iterator);
+      g_free (iterator->data);
+      g_list_free_1 (iterator);
+    }
+
+  userDN = dir_entry_list_getbyuid (cached_dir_entries, member);
+  if (userDN) {
+     g_assert(dir_entry_get_dn(userDN));
+     diradmin_group_add_dnmember(conn,dir_entry_get_dn(userDN));
+  }
+
+
+}
+
+void
+diradmin_group_remove_dnmember (diradmin_group * conn, gchar * member)
+{
+  GList *iterator = NULL;
+
+  g_assert(member);
+
+  iterator = g_list_find_custom (conn->diradmin_group_dnmembers,
+				 member, (GCompareFunc) g_strcasecmp);
+  if (iterator)
+    {
+      conn->diradmin_group_dnmembers =
+	g_list_remove_link (conn->diradmin_group_dnmembers, iterator);
+      g_free (iterator->data);
+      g_list_free_1 (iterator);
+    }
+
+  if (g_list_length (conn->diradmin_group_dnmembers) == 0)
+    diradmin_group_remove_objectclass (conn, "groupOfNames");
+
+}
+
+
+
 gboolean
 diradmin_group_has_member (diradmin_group * conn, gchar * allowedserver)
 {
   gchar *data;
   GList *iterator = NULL;
   iterator = g_list_first (conn->diradmin_group_members);
+  while (iterator)
+    {
+      data = iterator->data;
+      if (strcmp(allowedserver, iterator->data) == 0)
+	{
+	  return (TRUE);
+	}
+      iterator = g_list_next (iterator);
+    }
+  return (FALSE);
+}
+
+gboolean
+diradmin_group_has_dnmember (diradmin_group * conn, gchar * allowedserver)
+{
+  gchar *data;
+  GList *iterator = NULL;
+  iterator = g_list_first (conn->diradmin_group_dnmembers);
   while (iterator)
     {
       data = iterator->data;
@@ -196,7 +324,33 @@ diradmin_group_set_members (diradmin_group * conn, GList * memberlist)
   conn->diradmin_group_members = newmembers;
 }
 
+void
+diradmin_group_set_dnmembers (diradmin_group * conn, GList * memberlist)
+{
+  //removes all memberss in the list.  that means frees its allocated storage.
 
+  GList *newmembers = NULL;
+  GList *members = NULL;
+
+  //free the old members list
+  members = g_list_first (diradmin_group_get_dnmembers (conn));
+  while (members)
+    {
+      g_free (members->data);
+      members = g_list_next (members);
+    }
+  g_list_free (g_list_first (diradmin_group_get_dnmembers (conn)));
+
+  //make a copy of the passed members list into newmembers
+  members = g_list_first (memberlist);
+  while (members)
+    {
+      newmembers = g_list_append (newmembers, g_strdup (members->data));
+      members = g_list_next (members);
+    }
+
+  conn->diradmin_group_dnmembers = newmembers;
+}
 
 
 
@@ -212,10 +366,18 @@ diradmin_group_get_objectclasses (diradmin_group * conn)
 void
 diradmin_group_add_objectclass (diradmin_group * conn, gchar * allowedserver)
 {
+  int i;
+//  gboolean si = FALSE;
+
   diradmin_group_remove_objectclass (conn, allowedserver);
-  conn->diradmin_group_objectclasses =
-    g_list_append (conn->diradmin_group_objectclasses,
-		   g_strdup (allowedserver));
+
+  for (i = 0;group_handled_objectclasses[i];i++) {
+    if (g_strcasecmp(group_handled_objectclasses[i],allowedserver) == 0)
+      conn->diradmin_group_objectclasses =
+              g_list_append (conn->diradmin_group_objectclasses,
+			   g_strdup (allowedserver));
+
+  }
 }
 
 void
@@ -223,15 +385,17 @@ diradmin_group_remove_objectclass (diradmin_group * conn,
 				   gchar * allowedserver)
 {
   GList *iterator = NULL;
-  iterator = g_list_find_custom (conn->diradmin_group_objectclasses,
-				 allowedserver, (GCompareFunc) g_strcasecmp);
-  if (iterator)
-    {
-      conn->diradmin_group_objectclasses =
-	g_list_remove_link (conn->diradmin_group_objectclasses, iterator);
-      g_free (iterator->data);
-      g_list_free_1 (iterator);
-    }
+
+
+	for (iterator = g_list_first(conn->diradmin_group_objectclasses);iterator;iterator=iterator->next)
+		if (g_strcasecmp(iterator->data,allowedserver)==0)
+		{
+			conn->diradmin_group_objectclasses =
+			g_list_remove_link (conn->diradmin_group_objectclasses, iterator);
+			g_free (iterator->data);
+			g_list_free_1 (iterator);
+			iterator =g_list_first(conn->diradmin_group_objectclasses);
+		}
 }
 
 
@@ -253,20 +417,6 @@ diradmin_group_has_objectclass (diradmin_group * conn, gchar * allowedserver)
   return (FALSE);
 }
 
-void
-diradmin_group_dump_objectclasses (diradmin_group * conn)
-{
-  gchar *data;
-  GList *iterator = NULL;
-  iterator = g_list_first (conn->diradmin_group_objectclasses);
-  while (iterator)
-    {
-      data = iterator->data;
-      g_print ("\nGroup object class: %s", data);
-      iterator = g_list_next (iterator);
-    }
-}
-
 
 
 void
@@ -322,29 +472,29 @@ diradmin_group_new_from_ldap (connection_profile * usethisone, char *userdn)
   char **value_collection = NULL;
 
   gchar *filter;
+  gchar *locale_attr;
+  gchar *attribute;
+  BerElement *attributehandler;
 
   int i = 0;
 
-  gchar *attribute;
-  gchar *dn;
-  BerElement *attributehandler;
 
   user = g_new (diradmin_group, 1);
   user->diradmin_group_data = NULL;
   user->diradmin_group_members = NULL;
+  user->diradmin_group_dnmembers = NULL;
   user->diradmin_group_objectclasses = NULL;
 
 
   //check 4 connection
   g_assert (connection_profile_get_ldap_handler (usethisone));
 
-  filter = g_strconcat ("(&(objectClass=posixGroup)(cn=", userdn, "))", NULL);
+  filter = "objectclass=*";
 
   //look data up
   ldap_errors =
     ldap_search_s (connection_profile_get_ldap_handler (usethisone),
-		   connection_profile_get_treeroot (usethisone),
-		   LDAP_SCOPE_SUBTREE, filter, NULL, 0, &searchresults);
+		   userdn, LDAP_SCOPE_BASE, filter, NULL, 0, &searchresults);
   if (ldap_errors)
     {
 
@@ -354,6 +504,11 @@ diradmin_group_new_from_ldap (connection_profile * usethisone, char *userdn)
       g_print (": ");
       g_print (ldap_err2string (ldap_errors));
       g_print ("\n");
+      //ldap_msgfree (searchresults);
+	  if (ldap_errors ==  LDAP_SERVER_DOWN) {
+		connection_profile_invalidate(usethisone);
+	  }
+	  return NULL;
     }
   else
     {
@@ -364,10 +519,7 @@ diradmin_group_new_from_ldap (connection_profile * usethisone, char *userdn)
 
       // loop thru attribute values
 
-      dn =
-	ldap_get_dn (connection_profile_get_ldap_handler (usethisone), entry);
-      diradmin_group_set_attribute (user, "dn", dn);
-      g_free (dn);
+      diradmin_group_set_attribute (user, "dn", userdn);
 
 
       attribute =
@@ -381,30 +533,44 @@ diradmin_group_new_from_ldap (connection_profile * usethisone, char *userdn)
 	  value_collection =
 	    ldap_get_values (connection_profile_get_ldap_handler (usethisone),
 			     entry, attribute);
-	  g_print ("\n%s = %s\n", attribute, value_collection[0]);
+	  //g_print ("%s = %s\n", attribute, value_collection[0]);
 	  if (g_strcasecmp (attribute, "objectClass") == 0)
 	    {
-	      i = 0;
-	      while (value_collection[i])
+	      for (i = 0;value_collection[i];i++)
 		{
-		  g_print ("\n adding objectclass %s\n", value_collection[i]);
-		  diradmin_group_add_objectclass (user, value_collection[i]);
-		  i++;
+		    //g_print ("    adding objectclass %s\n", value_collection[i]);
+		    diradmin_group_add_objectclass (user, value_collection[i]);
 		}
 	    }
-	  else if (g_strcasecmp (attribute, "uniqueMember") != 0)
+	  else if (g_strcasecmp (attribute, "memberUid") == 0)
 	    {
-	      g_assert (value_collection[0]);
-	      diradmin_group_set_attribute (user, attribute,
-					    value_collection[0]);
+	      for (i = 0;value_collection[i];i++)
+                {
+                  locale_attr = convert_from_utf8 (value_collection[i]);
+			    //g_print ("    adding member%s\n", value_collection[i]);
+                  user->diradmin_group_members =
+                      g_list_append(user->diradmin_group_members,g_strdup(locale_attr));
+                  g_free(locale_attr);
+                }
+	    }
+	  else if (g_strcasecmp (attribute, "member") == 0)
+	    {
+	      for (i = 0;value_collection[i];i++)
+                {
+                  locale_attr = convert_from_utf8 (value_collection[i]);
+			    //g_print ("    adding DN member%s\n", value_collection[i]);
+		  diradmin_group_add_dnmember (user, locale_attr);
+                  g_free(locale_attr);
+                }
 	    }
 	  else
 	    {
-	      i = 0;
-	      while (value_collection[i])
+	      for (i = 0; value_collection[i]; i++)
 		{
-		  diradmin_group_add_member (user, value_collection[i]);
-		  i++;
+	     	  g_assert (value_collection[0]);
+            	  locale_attr = convert_from_utf8 (value_collection[0]);
+		  diradmin_group_set_attribute (user, attribute,locale_attr );
+                  g_free(locale_attr);
 		}
 	    }
 	  ldap_value_free (value_collection);
@@ -415,7 +581,6 @@ diradmin_group_new_from_ldap (connection_profile * usethisone, char *userdn)
 
       ldap_msgfree (searchresults);
     }
-  g_free (filter);
 
   return (user);
 }
@@ -466,7 +631,8 @@ gidnumber_to_cn (connection_profile * usethisone, gchar * gidnumber)
       g_assert (results);
       entry =
 	ldap_first_entry (connection_profile_get_ldap_handler
-			  (usethisone), results); if (entry == NULL)
+			  (usethisone), results);
+      if (entry == NULL)
 	{
 
 	  //any error?
@@ -505,13 +671,18 @@ cn_to_gidnumber (connection_profile * usethisone, gchar * cn)
   gchar *filter;
   gchar *attributetoreturn[2];
   gchar *toreturn = NULL;
+  gchar * utf8d_attr = NULL;
+
+  utf8d_attr = convert_to_utf8(cn);
+
   if (connection_profile_is_connected (usethisone) == FALSE)
     {
       g_print
 	("NOT CONNECTED!!!! - returning! (while finding out gidnumber_to_cn\n");
       return (NULL);
     }
-  filter = g_strconcat ("(&(objectClass=posixGroup)(cn=", cn, "))", NULL);
+  filter = g_strconcat ("(&(objectClass=posixGroup)(cn=", utf8d_attr, "))", 
+NULL);
   attributetoreturn[0] = "gidnumber";
   attributetoreturn[1] = NULL;
   //g_print(filter);
@@ -561,192 +732,111 @@ cn_to_gidnumber (connection_profile * usethisone, gchar * cn)
       ldap_msgfree (results);
     }
   g_free (filter);
+  g_free (utf8d_attr);
   return (toreturn);
 }
 
 
 
-LDAPMod **
+ldaptransaction*
 diradmin_group_generate_ldapdiff (diradmin_group *
 				  oldone, diradmin_group * newone)
 {
 
-  LDAPMod **diffs;
-  LDAPMod *curmod = NULL;
+  ldaptransaction *t;
   gchar **values;
-  gint curmod_counter = 0;
-  gchar *attributestocheck[] = {
-    NULL
-  };
-  GList *listone = NULL;
-  gint currattribute_counter = 0;
+  gchar *oldattr = NULL;
+  gchar *newattr = NULL;
+  GList *l = NULL;
+  gint acnt;
+
   g_assert (oldone);
   g_assert (newone);
-  diffs = g_new0 (LDAPMod *, sizeof (attributestocheck)
-		  + g_list_length (diradmin_group_get_members (oldone))
-		  + g_list_length (diradmin_group_get_members (newone))
-//               + g_list_length(diradmin_group_get_objectclasses (oldone))
-//               + g_list_length(diradmin_group_get_objectclasses (newone))
-    );
-  while (attributestocheck[currattribute_counter])
+  t = ldaptransaction_new();
+
+  for (acnt=0;group_handled_attributes[acnt];acnt++)
     {
-      if (g_strcasecmp
-	  (attributestocheck[currattribute_counter],
-	   "groupPassword") != 0
-	  && diradmin_group_get_attribute (newone,
-					   attributestocheck
-					   [currattribute_counter]) == NULL)
-	{
-	  curmod = g_new0 (LDAPMod, 1);
-	  curmod->mod_op = LDAP_MOD_DELETE;
-	  curmod->mod_type =
-	    g_strdup (attributestocheck[currattribute_counter]);
-	  curmod->mod_values = NULL;
-	  diffs[curmod_counter] = curmod;
-	  curmod_counter++;
-	}
-      else
-	if (diradmin_group_get_attribute
-	    (oldone,
-	     attributestocheck[currattribute_counter]) == NULL
-	    && diradmin_group_get_attribute (newone,
-					     attributestocheck
-					     [currattribute_counter]) != NULL)
+      oldattr = diradmin_group_get_attribute (oldone, group_handled_attributes[acnt]);
+      newattr = diradmin_group_get_attribute (newone, group_handled_attributes[acnt]);
+//      ispassword = g_strcasecmp  (group_handled_attributes[acnt], "groupPassword") == 0  ||  g_strcasecmp  (group_handled_attributes[acnt], "authPassword") == 0 ||  g_strcasecmp (group_handled_attributes[acnt], SAMBA_NT_PASSWORD) == 0 || g_strcasecmp (group_handled_attributes[acnt], SAMBA_LM_PASSWORD) == 0;
+      if (oldattr != NULL && g_strcasecmp(group_handled_attributes[acnt], "groupPassword") != 0
+	  && newattr == NULL) {
+	  g_print("- Replacing %s with NULL for %s.\n", oldattr, group_handled_attributes[acnt]);
+          ldaptransaction_delete(t,group_handled_attributes[acnt],NULL);
+      }
+      else if (oldattr == NULL && newattr != NULL)
 	{
 	  values = g_new0 (gchar *, 2);
-	  curmod = g_new0 (LDAPMod, 1);
-	  curmod->mod_op = LDAP_MOD_REPLACE;
-	  curmod->mod_type =
-	    g_strdup (attributestocheck[currattribute_counter]);
-	  values[0] =
-	    g_strdup (diradmin_group_get_attribute
-		      (newone, attributestocheck[currattribute_counter]));
-	  values[1] = NULL;
-	  curmod->mod_values = values;
-	  diffs[curmod_counter] = curmod;
-	  curmod_counter++;
+	  values[0] = convert_to_utf8(newattr);
+	  g_print("- Replacing NULL with %s for %s.\n", newattr, group_handled_attributes[acnt]);
+          ldaptransaction_replace(t, group_handled_attributes[acnt],values);
 	}
-      else
-	if (g_strcasecmp
-	    (diradmin_group_get_attribute
-	     (oldone,
-	      attributestocheck[currattribute_counter]),
-	     diradmin_group_get_attribute (newone,
-					   attributestocheck
-					   [currattribute_counter])) != 0)
+      else if (oldattr != NULL && newattr != NULL && strcmp(oldattr, newattr) != 0)
 	{
 	  values = g_new0 (gchar *, 2);
-	  curmod = g_new0 (LDAPMod, 1);
-	  curmod->mod_op = LDAP_MOD_REPLACE;
-	  curmod->mod_type =
-	    g_strdup (attributestocheck[currattribute_counter]);
-	  values[0] =
-	    g_strdup (diradmin_group_get_attribute
-		      (newone, attributestocheck[currattribute_counter]));
-	  values[1] = NULL;
-	  curmod->mod_values = values;
-	  diffs[curmod_counter] = curmod;
-	  curmod_counter++;
+	  values[0] = convert_to_utf8(newattr);
+	  g_print("- Replacing %s with %s for %s.\n", oldattr, newattr, group_handled_attributes[acnt]);
+          ldaptransaction_replace(t,group_handled_attributes[acnt],values);
 	}
-      currattribute_counter++;
     }
 
-  g_assert (diffs);
-  listone = diradmin_group_get_members (oldone);
-  while (listone)
-    {
-      if (diradmin_group_has_member (newone, listone->data) == FALSE)
+  for (l = diradmin_group_get_objectclasses (oldone);l;l=g_list_next(l))
+      if (diradmin_group_has_objectclass (newone, l->data) == FALSE)
 	{
 	  //generar mod de eliminacion
 	  values = g_new0 (gchar *, 2);
-	  curmod = g_new0 (LDAPMod, 1);
-	  curmod->mod_op = LDAP_MOD_DELETE;
-	  curmod->mod_type = g_strdup ("uniqueMember");
-	  values[0] = g_strdup (listone->data);
-	  values[1] = NULL;
-	  curmod->mod_values = values;
-	  diffs[curmod_counter] = curmod;
-	  curmod_counter++;
+	  values[0] = g_strdup (l->data);
+          ldaptransaction_delete(t,"objectClass",values);
 	}
-      listone = g_list_next (listone);
-    }
-  listone = diradmin_group_get_members (newone);
-  while (listone)
-    {
-      if (diradmin_group_has_member (oldone, listone->data) == FALSE)
+  for (l = diradmin_group_get_objectclasses (newone);l;l=g_list_next(l))
+      if (diradmin_group_has_objectclass (oldone, l->data) == FALSE)
 	{
 	  //generar mod de eliminacion
 	  values = g_new0 (gchar *, 2);
-	  curmod = g_new0 (LDAPMod, 1);
-	  curmod->mod_op = LDAP_MOD_ADD;
-	  curmod->mod_type = g_strdup ("uniqueMember");
-	  values[0] = g_strdup (listone->data);
-	  values[1] = NULL;
-	  curmod->mod_values = values;
-	  diffs[curmod_counter] = curmod;
-	  curmod_counter++;
+	  values[0] = g_strdup (l->data);
+          ldaptransaction_add(t,"objectClass",values);
 	}
-      listone = g_list_next (listone);
-    }
-
-  listone = diradmin_group_get_objectclasses (newone);
-  while (listone)
-    {
-      if (diradmin_group_has_objectclass (oldone, listone->data) == FALSE)
+  for (l = diradmin_group_get_members (oldone);l;l=g_list_next(l))
+      if (diradmin_group_has_member (newone, l->data) == FALSE)
+	{
+	  //generar mod de eliminacion
+	  values = g_new0 (gchar *, 2);
+	  values[0] = convert_to_utf8( l->data);
+          ldaptransaction_delete(t,"memberUid",values);
+	}
+  for (l = diradmin_group_get_members (newone);l;l=g_list_next(l))
+      if (diradmin_group_has_member (oldone, l->data) == FALSE)
 	{
 	  //generar mod de adicion
 	  values = g_new0 (gchar *, 2);
-	  curmod = g_new0 (LDAPMod, 1);
-	  curmod->mod_op = LDAP_MOD_ADD;
-	  curmod->mod_type = g_strdup ("objectclass");
-	  values[0] = g_strdup (listone->data);
-	  values[1] = NULL;
-	  curmod->mod_values = values;
-	  diffs[curmod_counter] = curmod;
-	  curmod_counter++;
+	  values[0] = convert_to_utf8( l->data);
+          ldaptransaction_add(t,"memberUid",values);
+	}
+  for (l = diradmin_group_get_dnmembers (oldone);l;l=g_list_next(l))
+      if (diradmin_group_has_dnmember (newone, l->data) == FALSE)
+	{
+	  //generar mod de eliminacion
+	  values = g_new0 (gchar *, 2);
+	  values[0] = convert_to_utf8( l->data);
+          ldaptransaction_delete(t,"member",values);
+	}
+  for (l = diradmin_group_get_dnmembers (newone);l;l=g_list_next(l))
+      if (diradmin_group_has_dnmember (oldone, l->data) == FALSE)
+	{
+	  //generar mod de adicion
+	  values = g_new0 (gchar *, 2);
+	  values[0] = convert_to_utf8( l->data);
+          ldaptransaction_add(t,"member",values);
 	}
-      listone = g_list_next (listone);
-    }
-  if (diradmin_group_has_objectclass (newone, "groupOfUniqueNames") == FALSE
-      &&
-      diradmin_group_has_objectclass (oldone, "groupOfUniqueNames") == TRUE)
-    {
-
-      //generar mod de eliminacion de este object class
-      values = g_new0 (gchar *, 2);
-      curmod = g_new0 (LDAPMod, 1);
-      curmod->mod_op = LDAP_MOD_DELETE;
-      curmod->mod_type = g_strdup ("objectClass");
-      values[0] = g_strdup ("groupOfUniqueNames");
-      values[1] = NULL;
-      curmod->mod_values = values;
-      diffs[curmod_counter] = curmod;
-      curmod_counter++;
-
-    }
 
 
-  if (curmod_counter == 0)
-    {
-      g_free (diffs);
-      diffs = NULL;
-    }
-  else
-    diffs[curmod_counter] = NULL;
-
-  g_print ("\nPrinting objectclasses from the old one:\n");
-  diradmin_group_dump_objectclasses (oldone);
-  g_print ("\nPrinting objectclasses from the new one:\n");
-  diradmin_group_dump_objectclasses (newone);
-
-  return diffs;
+  return t;
 }
 
 diradmin_group *
 create_group_struct_from_dialogbox (connection_profile
 				    * conn, GtkWidget * dialogbox, gchar * dn)
 {
-  //TODO: TRICK TO GET GECOS TO BE WRITTEN HERE!!!!!!
 
   diradmin_group *newgroup;
   gint iterator;
@@ -754,116 +844,141 @@ create_group_struct_from_dialogbox (connection_profile
   gchar *gidnumber = NULL;
   GtkCList *members = NULL;
   gchar *member = NULL;
-  GList * listone;
-  
-  members = (GtkCList *) lookup_widget (dialogbox, "members");
-  cn = gtk_entry_get_text (GTK_ENTRY (lookup_widget (dialogbox, "cn")));
-  gidnumber =
-    gtk_entry_get_text (GTK_ENTRY (lookup_widget (dialogbox, "gidnumber")));
-  newgroup = diradmin_group_new (dn, cn, gidnumber, NULL);
+  gchar *membertype = NULL;
+  gchar*membercn = NULL;
+  GList *listone;
+  dir_entry*memberentry;
+  gint thisgidnumber;
+  gchar *sambaDomainName = NULL;
+  gchar *sambaDomainMapping = NULL;
+  gchar *sambaSID = NULL;
+
+	members = (GtkCList *) lookup_widget (dialogbox, "members");
+	cn = gtk_entry_get_text (GTK_ENTRY (lookup_widget (dialogbox, "cn")));
+	gidnumber =
+		gtk_entry_get_text (GTK_ENTRY (lookup_widget (dialogbox, "gidnumber")));
+	thisgidnumber = atoi(gidnumber);
+
+  if (GTK_TOGGLE_BUTTON
+      (lookup_widget (dialogbox, "sambaGroupMapping"))->active)
+    {
+      gchar *warning = NULL;
+      sambaDomainName =
+        gtk_entry_get_text (GTK_ENTRY (lookup_widget (dialogbox, "sambaDomainName")));
+      sambaDomainMapping =
+        gtk_entry_get_text (GTK_ENTRY (lookup_widget (dialogbox, "sambaDomainMapping")));
+      sambaSID = app_get_sambasid_from_ldap(conn, sambaDomainName, app_get_sambaridfromgroupmapping(sambaDomainMapping), gidnumber, 1, &warning);
+      if (warning) {
+        g_print(warning);
+      }
+    }
+
+  newgroup = diradmin_group_new (dn, cn, gidnumber, sambaSID);
   for (iterator = 0; iterator < members->rows; iterator++)
     {
-      gtk_clist_get_text (members, iterator, 0, &member);
-      diradmin_group_add_member (newgroup, member);
+		member = gtk_clist_get_row_data(members,iterator);
+		gtk_clist_get_text (members, iterator, 0, &membertype);
+
+		//handle case donde se cambia el primary group y despues recien se da click en add members
+		memberentry = cached_dir_entries_getbydn(member);
+		if (memberentry
+			&& g_strcasecmp(membertype,"Primary") == 0
+			&& memberentry->gidnumber != thisgidnumber)
+		{
+			g_print("      User %s is no longer primary, dropping\n",member);
+		} else {
+			g_print("      Appending to dnmembers %s member %s\n",membertype,member);
+			diradmin_group_add_dnmember (newgroup, member);
+		}
+		//solo si el usuario existe todavia en la base de datos!
+		if (memberentry && dir_entry_is_user(memberentry)) {
+			if (g_strcasecmp(membertype,"Secondary") == 0) {
+				membercn = dir_entry_get_uid(memberentry);
+				g_print("      Appending to members %s member %s\n",membertype,membercn);
+				diradmin_group_add_member(newgroup,membercn);
+			}
+		}
     }
 
   listone = diradmin_group_get_objectclasses (newgroup);
   while (listone)
     {
-     g_print(listone->data); listone=listone->next;
+      g_print (" o Including objectclass %s\n", listone->data);
+      listone = listone->next;
     }
 
+
   return (newgroup);
 }
 
 
 
-LDAPMod **
+ldaptransaction *
 diradmin_group_create_ldapdiff (diradmin_group * newone)
 {
 
-  LDAPMod **diffs;
-  LDAPMod *curmod = NULL;
+  ldaptransaction * t;
+  GList *l = NULL;
+  gint acnt = 0;
+  dir_entry *userDN = NULL;
   gchar **values;
-  gint curmod_counter = 0;
-  gchar *attributestocheck[] = {
-    "cn", "gidnumber", NULL
-  };
-  GList *listone = NULL;
-  gint currattribute_counter = 0;
-
+      
   g_assert (newone);
-  diffs = g_new0 (LDAPMod *, sizeof (attributestocheck)
-		  + g_list_length (diradmin_group_get_members (newone))
-		  +
-		  g_list_length (diradmin_group_get_objectclasses (newone)));
+  t = ldaptransaction_new();
 
-  values = g_new0 (gchar *, 7);
-  curmod = g_new0 (LDAPMod, 1);
-  curmod->mod_op = LDAP_MOD_ADD;
-  curmod->mod_type = g_strdup ("objectClass");
-  values[0] = g_strdup ("top");
-  values[1] = g_strdup ("posixGroup");
-//  values[2] = g_strdup ("groupOfUniqueNames");
-  values[3] = NULL;
-  curmod->mod_values = values;
-  diffs[curmod_counter] = curmod;
-  curmod_counter++;
-  while (attributestocheck[currattribute_counter])
+//  values = g_new0 (gchar *, 7);
+//  values[0] = g_strdup ("top");
+//  values[1] = g_strdup ("posixGroup");
+//  if (g_list_length (diradmin_group_get_members (newone)))
+//    values[2] = g_strdup ("groupOfNames");
+  l = diradmin_group_get_objectclasses (newone);
+  values = g_new0(gchar*,g_list_length(l)+1);
+  acnt = 0;
+  for (l = g_list_first(l); l; l = g_list_next (l)) {
+    values[acnt] = g_strdup (l->data); acnt++;
+  }
+  ldaptransaction_add(t,"objectclass",values);
+
+  for(acnt=0;group_handled_attributes[acnt];acnt++)
     {
       if (diradmin_group_get_attribute
-	  (newone, attributestocheck[currattribute_counter]) != NULL)
+	  (newone, group_handled_attributes[acnt]) != NULL)
 	{
 	  values = g_new0 (gchar *, 2);
-	  curmod = g_new0 (LDAPMod, 1);
-	  curmod->mod_op = LDAP_MOD_ADD;
-	  curmod->mod_type =
-	    g_strdup (attributestocheck[currattribute_counter]);
-	  values[0] =
-	    g_strdup (diradmin_group_get_attribute
-		      (newone, attributestocheck[currattribute_counter]));
-	  values[1] = NULL;
-	  curmod->mod_values = values;
-	  diffs[curmod_counter] = curmod;
-	  curmod_counter++;
+	  values[0] = convert_to_utf8
+		(diradmin_group_get_attribute(newone, group_handled_attributes[acnt]));
+          ldaptransaction_add(t,group_handled_attributes[acnt],values);
 	}
-      currattribute_counter++;
     }
 
-  g_assert (diffs);
-  if (g_list_length(diradmin_group_get_members(newone)))
-   {
-    values = g_new0 (gchar *, 2);
-    curmod = g_new0 (LDAPMod, 1);
-    curmod->mod_op = LDAP_MOD_ADD;
-    curmod->mod_type = g_strdup ("objectClass");
-    values[0] = g_strdup ("groupOfUniqueNames");
-    values[1] = NULL;
-    curmod->mod_values = values;
-    diffs[curmod_counter] = curmod;
-    curmod_counter++;
-
-    listone = diradmin_group_get_members (newone);
-    while (listone)
-     {
-      values = g_new0 (gchar *, 2);
-      curmod = g_new0 (LDAPMod, 1);
-      curmod->mod_op = LDAP_MOD_ADD;
-      curmod->mod_type = g_strdup ("uniqueMember");
-      values[0] = g_strdup (listone->data);
-      values[1] = NULL;
-      curmod->mod_values = values;
-      diffs[curmod_counter] = curmod;
-      curmod_counter++;
-      listone = g_list_next (listone);
-     }
-   }
-  if (curmod_counter == 0)
+  if (g_list_length (diradmin_group_get_members (newone)))
     {
-      g_free (diffs);
-      diffs = NULL;
+    values = g_new0 (gchar *, 2 + g_list_length(diradmin_group_get_members (newone)));
+    acnt = 0;
+    for(l = diradmin_group_get_members (newone);l;l=g_list_next(l))
+	{
+	  values[acnt] =
+	   convert_to_utf8( l->data);
+          acnt++;
+	}
+    ldaptransaction_add(t,"memberUid",values);
+
+    /* Generating DN list for groupOfNames */
+    values = g_new0 (gchar *, 2 + g_list_length(diradmin_group_get_members (newone)));
+    acnt = 0;
+    for (l = diradmin_group_get_members (newone);l;l=g_list_next(l))
+	{
+          userDN = dir_entry_list_getbyuid (cached_dir_entries, l->data);
+	  if (userDN) {
+            g_print ("\n\n\n\n\n\n\n%s",dir_entry_get_dn(userDN));
+	    values[acnt] = g_strdup(dir_entry_get_dn(userDN));
+            acnt++;
+	  }
+	}
+    ldaptransaction_add(t,"member",values);
+
     }
-  else
-    diffs[curmod_counter] = NULL;
-  return diffs;
+
+
+  return t;
 }
